@@ -1042,15 +1042,14 @@ static uint_t Cookies_internal_dots_required(const char *host)
 
       if (tld_len > 0) {
          /* These TLDs were chosen by examining the current publicsuffix list
-          * in October 2012 and picking out those where it was simplest for
+          * in February 2014 and picking out those where it was simplest for
           * them to describe the situation by beginning with a "*.[tld]" rule
           * or every rule was "[something].[tld]".
           */
-         const char *const tlds[] = {"ar","au","bd","bn","ck","cy","er","et",
-                                     "fj","fk","gt","gu","il","jm","ke","kh",
-                                     "kp","kw","lb","lr","mm","mt","mz","ni",
-                                     "np","nz","om","pg","py","sv","tr","uk",
-                                     "ve","ye","za","zm","zw"};
+         const char *const tlds[] = {"bd","bn","ck","cy","er","et","fj","fk",
+                                     "gu","il","jm","ke","kh","kw","mm","mz",
+                                     "ni","np","nz","pg","tr","uk","ye","za",
+                                     "zm","zw"};
          uint_t i, tld_num = sizeof(tlds) / sizeof(tlds[0]);
 
          for (i = 0; i < tld_num; i++) {
