@@ -230,70 +230,28 @@ const char *const AboutSplash=
 "<tr>\n"
 " <td bgcolor='#CCCCCC'>\n"
 "  <h4>Release overview</h4>\n"
-"  April 09, 2014\n"
+"  December 24, 2014\n"
 "<tr>\n"
 " <td bgcolor='#FFFFFF'>\n"
 "  <table border='0' cellspacing='0' cellpadding='5'>\n"
 "  <tr>\n"
 "   <td>\n"
 "<p>\n"
-"dillo-3.0.4 adds some nice new features, as listed below.\n"
-"<p>\n"
-"This release comes with better <b>text rendering</b> in the form of\n"
-"some linebreaking fixes, and optimization for non-justified text,\n"
-"including a new preference stretchability_factor.\n"
-"<p>\n"
-"It also comes with support for a few <b>HTML5</b> elements and better CSS\n"
-"including background <b>images</b> (enable load_background_images in\n"
-"dillorc or the tools menu).\n"
-"<p>\n"
-"It has enhanced <b>security</b> by fixing a set of potentially exploitable\n"
-"code patterns reported by the Oulu University Secure Programming Group.\n"
-"<p>\n"
-"Finally it also fixes compiling on HURD and IRIX.\n"
-"<p>\n"
-"The core team welcomes developers willing to join our workforce.\n"
-"<p>\n"
-"  </table>\n"
-"</table>\n"
-"</table>\n"
-"\n"
-"<br>\n"
-"\n"
-"<table border='0' cellpadding='0' cellspacing='0' align='center' bgcolor='#000000' width='100%'><tr><td>\n"
-"<table border='0' cellpadding='5' cellspacing='1' width='100%'>\n"
-"<tr>\n"
-" <td bgcolor='#CCCCCC'>\n"
-"  <h4>ChangeLog highlights</h4>\n"
-"  (Extracted from the\n"
-"  <a href='http://hg.dillo.org/dillo/raw-file/tip/ChangeLog'>full\n"
-"  ChangeLog</a>)\n"
-"<tr>\n"
-" <td bgcolor='#FFFFFF'>\n"
-"  <table border='0' cellspacing='0' cellpadding='5'>\n"
-"  <tr>\n"
-"   <td>\n"
+"The dillo-3.0.4.1 release brings you fixes:\n"
 "<ul>\n"
-"<li> OPTGROUP and INS elements.\n"
-"<li> Added some HTML5 elements.\n"
-"<li> Added <b>show_ui_tooltip</b> preference (BUG#1140).\n"
-"<li> Make embedding into other applications more reliable (BUG#1127).\n"
-"<li> Add search from address bar.\n"
-"<li> Better scaling (down) of images, even with consideration of gamma \n"
-" correction.\n"
-"<li> Some linebreaking fixes, and optimization for non-justified text,\n"
-" including new preference <b>stretchability_factor</b>.\n"
-"<li> Added <b>white_bg_replacement</b> preference.\n"
-"<li> Implemented background images (except 'background-attachment'), added\n"
-" <b>load_background_images</b> preference, as well as a new entry in the\n"
-" tools menu.\n"
-"<li> Fix a set of bugs reported by Oulu Univ. Secure Programming Group\n"
-" (HTML parsing, URL resolution, GIF processing, etc.)\n"
-"<li> Made <b>show_url</b> dillorc option work again (BUG#1128)\n"
-"<li> Fix compiling on Hurd.\n"
-"<li> Avoid Dpid children becoming zombies.\n"
-"<li> Fix compiling on IRIX with MIPSpro compiler.\n"
+"<li> for linking with the recently-released fltk-1.3.3\n"
+" (we don't use <tt>fl_oldfocus</tt> anymore).\n"
+"<li> to make sure that windows are resizable with fltk-1.3.3.\n"
+"<li> not to load background images, or follow redirections or meta refresh,\n"
+"     in <tt>--local</tt> mode (security).\n"
+"<li> to permit linking on OS X (remove our <tt>Fl_Printer</tt> stub).\n"
+"<li> for a crash when searching from the address bar and no search urls are\n"
+"     found in dillorc.\n"
 "</ul>\n"
+"<p>\n"
+"...that shouldn't have to wait until dillo-3.1 is ready with its floating\n"
+"elements and assorted good things.\n"
+"<p>\n"
 "  </table>\n"
 "</table>\n"
 "</table>\n"
