@@ -287,13 +287,15 @@ static inline size_t Gif_data_blocks(const uchar_t *Buf, size_t BSize)
  */
 static inline size_t Gif_do_generic_ext(const uchar_t *Buf, size_t BSize)
 {
-   size_t Size = Buf[0] + 1, DSize;
+
+   size_t Size = Buf[0] + 1,  /* (uchar_t + 1) can't overflow size_t */
+          DSize;
 
    /* The Block size (the first byte) is supposed to be a specific size
     * for each extension... we don't check.
     */
 
-   if (Buf[0] > BSize)
+   if (Size > BSize)
       return 0;
    DSize = Gif_data_blocks(Buf + Size, BSize - Size);
    if (!DSize)
@@ -814,8 +816,10 @@ static size_t Gif_do_img_desc(DilloGif *gif, void *Buf,
       return 0;
    }
 
+   /** \todo Gamma for GIF? */
    a_Dicache_set_parms(gif->url, gif->version, gif->Image,
-                       gif->Width, gif->Height, DILLO_IMG_TYPE_INDEXED);
+                       gif->Width, gif->Height, DILLO_IMG_TYPE_INDEXED,
+                       1 / 2.2);
 
    Flags = buf[8];
 
