@@ -53,6 +53,7 @@ int PrefsParser::parseOption(char *name, char *value)
    /* Symbol array, sorted alphabetically */
    const SymNode_t symbols[] = {
       { "allow_white_bg", &prefs.allow_white_bg, PREFS_BOOL },
+      { "white_bg_replacement", &prefs.white_bg_replacement, PREFS_COLOR },
       { "bg_color", &prefs.bg_color, PREFS_COLOR },
       { "buffered_drawing", &prefs.buffered_drawing, PREFS_INT32 },
       { "contrast_visited_color", &prefs.contrast_visited_color, PREFS_BOOL },
@@ -78,6 +79,7 @@ int PrefsParser::parseOption(char *name, char *value)
       { "http_user_agent", &prefs.http_user_agent, PREFS_STRING },
       { "limit_text_width", &prefs.limit_text_width, PREFS_BOOL },
       { "load_images", &prefs.load_images, PREFS_BOOL },
+      { "load_background_images", &prefs.load_background_images, PREFS_BOOL },
       { "load_stylesheets", &prefs.load_stylesheets, PREFS_BOOL },
       { "middle_click_drags_page", &prefs.middle_click_drags_page,
         PREFS_BOOL },
@@ -102,10 +104,12 @@ int PrefsParser::parseOption(char *name, char *value)
       { "show_quit_dialog", &prefs.show_quit_dialog, PREFS_BOOL },
       { "show_reload", &prefs.show_reload, PREFS_BOOL },
       { "show_save", &prefs.show_save, PREFS_BOOL },
+      { "show_url", &prefs.show_url, PREFS_BOOL },
       { "show_search", &prefs.show_search, PREFS_BOOL },
       { "show_stop", &prefs.show_stop, PREFS_BOOL },
       { "show_tools", &prefs.show_tools, PREFS_BOOL },
       { "show_tooltip", &prefs.show_tooltip, PREFS_BOOL },
+      { "show_ui_tooltip", &prefs.show_ui_tooltip, PREFS_BOOL },
       { "small_icons", &prefs.small_icons, PREFS_BOOL },
       { "start_page", &prefs.start_page, PREFS_URL },
       { "theme", &prefs.theme, PREFS_STRING },
@@ -127,6 +131,8 @@ int PrefsParser::parseOption(char *name, char *value)
       { "penalty_em_dash_right", &prefs.penalty_em_dash_right,
         PREFS_FRACTION_100 },
       { "penalty_em_dash_right_2", &prefs.penalty_em_dash_right_2,
+        PREFS_FRACTION_100 },
+      { "stretchability_factor", &prefs.stretchability_factor,
         PREFS_FRACTION_100 }
    };
 
